% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_overview.R
\name{data_overview}
\alias{data_overview}
\title{Data Overview}
\usage{
data_overview(data)
}
\arguments{
\item{data}{A data.frame.}
}
\value{
An object of class \code{DataOverview} containing
dataset-level summary information.
}
\description{
Provides a concise overview of the size, structure,
completeness, and variable types of a dataset.
}
\details{
The function reports the number of rows and variables,
total number of cells, missing and complete cells,
completeness percentage, complete cases, duplicate rows,
and counts of common variable types.
}
\examples{
data_overview(iris)

dat <- iris
dat$Sepal.Length[1:5] <- NA
data_overview(dat)

}
