% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print_DataAuditRuleResult.R
\name{print.DataAuditRuleResult}
\alias{print.DataAuditRuleResult}
\title{Print a DataAudit Rule Result}
\usage{
\method{print}{DataAuditRuleResult}(x, ...)
}
\arguments{
\item{x}{An object of class \code{DataAuditRuleResult}.}

\item{...}{Additional arguments, currently unused.}
}
\value{
The input object \code{x}, invisibly.
}
\description{
Prints a concise summary of custom audit-rule results
produced by \code{\link{apply_audit_rules}}.
}
