% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print_DataAuditReport.R
\name{print.DataAuditReport}
\alias{print.DataAuditReport}
\title{Print a DataAudit Report}
\usage{
\method{print}{DataAuditReport}(x, ...)
}
\arguments{
\item{x}{An object of class \code{DataAuditReport}.}

\item{...}{Additional arguments, currently unused.}
}
\value{
The input object \code{x}, invisibly.
}
\description{
Prints a concise summary of a data audit produced by
\code{\link{audit_data}}.
}
