% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/audit_score.R
\name{audit_score}
\alias{audit_score}
\title{Calculate a Data Quality Score}
\usage{
audit_score(x)
}
\arguments{
\item{x}{An object of class \code{DataAuditReport}.}
}
\value{
An object of class \code{DataAuditScore}.
}
\description{
Calculates a standardized data-quality score from a
\code{DataAuditReport} produced by \code{\link{audit_data}}.
}
\details{
The score ranges from 0 to 100, where higher values
indicate better data quality.

The score is based on automatic audit issues and custom
rule violations relative to the number of observations
and variables in the audited dataset.
}
\examples{
dat <- data.frame(
  ID = c(1, 2, 2, 4),
  Value = c(10, NA, Inf, 20)
)

report <- audit_data(dat)
audit_score(report)

}
