% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/missing_check.R
\name{missing_check}
\alias{missing_check}
\title{Check Missing Values}
\usage{
missing_check(data)
}
\arguments{
\item{data}{A data.frame.}
}
\value{
A data.frame of class \code{MissingCheck}
containing variable-level missing-value information.
}
\description{
Examines each variable in a dataset and reports the
number and percentage of missing and complete values.
}
\details{
For each variable, the function reports the total number
of observations, number of missing values, number of
complete values, and their corresponding percentages.

Missing values are identified using \code{is.na()}.
}
\examples{
missing_check(iris)

dat <- iris
dat$Sepal.Length[1:10] <- NA
dat$Species[1:5] <- NA
missing_check(dat)

}
