% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print_DataAuditScore.R
\name{print.DataAuditScore}
\alias{print.DataAuditScore}
\title{Print a DataAudit Score}
\usage{
\method{print}{DataAuditScore}(x, ...)
}
\arguments{
\item{x}{An object of class \code{DataAuditScore}.}

\item{...}{Additional arguments, currently unused.}
}
\value{
The input object \code{x}, invisibly.
}
\description{
Prints a concise summary of a data-quality score
generated by \code{\link{audit_score}}.
}
