% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary_DataAuditRuleResult.R
\name{summary.DataAuditRuleResult}
\alias{summary.DataAuditRuleResult}
\title{Summarize a DataAudit Rule Result}
\usage{
\method{summary}{DataAuditRuleResult}(object, ...)
}
\arguments{
\item{object}{An object of class
\code{DataAuditRuleResult}.}

\item{...}{Additional arguments, currently unused.}
}
\value{
An object of class
\code{summary.DataAuditRuleResult}.
}
\description{
Produces a compact summary of custom validation-rule
results generated by \code{\link{apply_audit_rules}}.
}
